#define B_CALL(xxxx) rst 28h \ .dw xxxx
#define B_JUMP(xxxx) call 50h \ .dw xxxx

Delay     =    $28
Cursor    =    $E2

;         Input:
;A        RightBound
;DE       StringLocation
;(cur...) LeftBound

;         Output Mode:   Output Enter:
;A        $00            $FF
;F(C)     1              0
;F(Z)     0              1

;***************************************
     
CLL_INPUTSTR:
     ld   (DAT_RightBound),a
     ld   a,(curcol)
     ld   (DAT_LeftBound),a
     
     res  textinverse,(IY+textflags)
     XOR  a
     ld   (IY+indicflags),a
     
;***************************************
     
CLL_IS_PreGetCSC:
     ld   a,Cursor
     bit  curon,(IY+curflags)
     set  curon,(IY+curflags)
     jr   z,CLL_IS_CurOn
     
     ld   a,(de)
     res  curon,(IY+curflags)
     
     OR   a
     jr   nz,CLL_IS_CurOn
     
     ld   a,$20
     
CLL_IS_CurOn:
     B_CALL(_putmap)
     ld   b,Delay
     
;***************************************
     
CLL_IS_GetCSC:
     EI
     HALT
     B_CALL(_getcsc)
     OR   a
     jr   nz,CLL_IS_KeyPress
     DJNZ CLL_IS_GetCSC
     jr   CLL_IS_PreGetCSC
     
;***************************************
     
CLL_IS_KeyPress:
     res  curon,(IY+curflags)
     ld   bc,DAT_KeyCodes_End - DAT_KeyCodes + 1
     ld   hl,DAT_KeyCodes_End - 1
     
     CPDR
     
     ld   a,c
     OR   a
     jr   z,CLL_IS_PreGetCSC
     
     SUB  2
     RET  c
     RET  z
     
     SUB  2
     jr   c,CLL_IS_Delete
     jr   z,CLL_IS_Clear
     
     ld   b,a
     ld   a,(de)
     OR   a
     jr   nz,CLL_IS_NoZero
     
     ld   a,$20
     
CLL_IS_NoZero:
     B_CALL(_putmap)
     ld   a,b
     
     SUB  2
     jr   c,CLL_IS_Right
     jr   z,CLL_IS_Down
     
     SUB  2
     jr   c,CLL_IS_Left
     jp   z,CLL_IS_Up
     
     ADD  a,$40
     B_CALL(_putmap)
     ld   (de),a
     
;***************************************
     
CLL_IS_Right:
     ld   a,(de)
     OR   a
     jr   z,CLL_IS_PreGetCSC
     
     ld   a,(DAT_RightBound)
     ld   hl,curcol
     cp  (hl)
     jr   z,CLL_IS_PreGetCSC
     
     INC  (hl)
     INC  de
     jr   CLL_IS_PreGetCSC
     
;***************************************
     
CLL_IS_Down:
     ld   b,a
     ld   a,(DAT_RightBound)
     ld   c,a
     ld   hl,curcol
     
CLL_IS_DownLoop:
     ld   a,(de)
     OR   a
     jr   z,CLL_IS_PreGetCSC
     ld   a,c
     cp   (hl)
     jr   z,CLL_IS_PreGetCSC
     INC  (hl)
     INC  de
     jr   CLL_IS_DownLoop
     
;***************************************
     
CLL_IS_Left:
     ld   a,(DAT_LeftBound)
     ld   hl,curcol
     cp  (hl)
     jp   z,CLL_IS_PreGetCSC
     
     DEC  (hl)
     DEC  de
     jp   CLL_IS_PreGetCSC
     
;***************************************
     
CLL_IS_Delete:
     
     push de
     
     ld   a,(DAT_RightBound)
     ld   hl,curcol
     ld   c,(hl)
     SUB  (hl)
     ld   b,a
     jr   z,CLL_IS_DeleteLC
     
     ld   l,e
     ld   h,d
     INC  hl
     
CLL_IS_DeleteLoop:
     ld   a,(hl)
     ld   (de),a
     OR   a
     jr   nz,CLL_IS_DeleteNoSpace
     
     ld   a,$20
     
CLL_IS_DeleteNoSpace:
     B_CALL(_putc)
     INC  de
     INC  hl
     DJNZ CLL_IS_DeleteLoop
     XOR  a
     
CLL_IS_DeleteLC:
     ld   (de),a
     ld   a,$20
     B_CALL(_putmap)
     ld   a,c
     ld   (curcol),a
     pop  de
     jp   CLL_IS_PreGetCSC
     
;***************************************
     
CLL_IS_Clear:
     ld   a,(DAT_LeftBound)
     ld   b,a
     ld   hl,curcol
     SUB  (hl)
     ld   (hl),b
     ld   c,a
     ld   b,$FF
     EX   de,hl
     ADD  hl,bc
     EX   de,hl
     ld   a,(DAT_RightBound)
     ld   c,a
     
CLL_IS_ClearLoop:
     XOR  a
     ld   (de),a
     
     ld   a,c
     cp   (hl)
     ld   a,$20
     jr   z,CLL_IS_ClearLC
     
     INC  de
     B_CALL(_putc)
     
     jr   CLL_IS_ClearLoop
     
CLL_IS_ClearLC:
     B_CALL(_putmap)
     
;***************************************
     
CLL_IS_Up:
     ld   a,(DAT_LeftBound)
     ld   b,a
     ld   hl,curcol
     SUB  (hl)
     ld   (hl),b
     ld   l,a
     ld   h,$FF
     ADD  hl,de
     EX   de,hl
     jp   CLL_IS_PreGetCSC
     
;***************************************
     
DAT_LeftBound:
     .db  $00
     
DAT_RightBound:
     .db  $00
     
DAT_KeyCodes:
     .db  skenter             ;<Enter>
     .db  skmode              ;<Mode>
     .db  skdel               ;<Delete>
     .db  skclear             ;<Clear>
     
     .db  skright             ;<Right>
     .db  skdown              ;<Down>
     .db  skleft              ;<Left>
     .db  skup                ;<Up>
     
     .db  $2F,$27,$1F         ;A B C
     .db  $2E,$26,$1E,$16,$0E ;D E F G H
     .db  $2D,$25,$1D,$15,$0D ;I J K L M
     .db  $2C,$24,$1C,$14,$0C ;N O P Q R
     .db  $2B,$23,$1B,$13,$0B ;S T U V W
     .db  $2A,$22,$1A         ;X Y Z
DAT_KeyCodes_End:
     
     .end
END
